@@ -29,7 +29,7 @@ pthread_mutex_t data_mutex = {0};
 int snac_upgrade(xs_str **error);
 
 
-int srv_open(char *basedir, int auto_upgrade)
+int srv_open(const char *basedir, int auto_upgrade)
 /* opens a server */
 {
     int ret = 0;
@@ -58,10 +58,10 @@ int srv_open(char *basedir, int auto_upgrade)
         if (srv_config == NULL)
             error = xs_fmt("ERROR: cannot parse '%s'", cfg_file);
         else {
-            char *host;
-            char *prefix;
-            char *dbglvl;
-            char *proto;
+            const char *host;
+            const char *prefix;
+            const char *dbglvl;
+            const char *proto;
 
             host   = xs_dict_get(srv_config, "host");
             prefix = xs_dict_get(srv_config, "prefix");
@@ -114,7 +114,7 @@ int srv_open(char *basedir, int auto_upgrade)
 #endif
 
 #ifdef __OpenBSD__
-    char *v = xs_dict_get(srv_config, "disable_openbsd_security");
+    const char *v = xs_dict_get(srv_config, "disable_openbsd_security");
 
     if (v && xs_type(v) == XSTYPE_TRUE) {
         srv_debug(1, xs_dup("OpenBSD security disabled by admin"));
@@ -193,7 +193,7 @@ int user_open(snac *user, const char *uid)
             xs *lcuid = xs_tolower_i(xs_dup(uid));
             xs *ulist = user_list();
             xs_list *p = ulist;
-            xs_str *v;
+            const xs_str *v;
 
             while (xs_list_iter(&p, &v)) {
                 xs *v2 = xs_tolower_i(xs_dup(v));
@@ -289,7 +289,7 @@ int user_open_by_md5(snac *snac, const char *md5)
 {
     xs *ulist  = user_list();
     xs_list *p = ulist;
-    xs_str *v;
+    const xs_str *v;
 
     while (xs_list_iter(&p, &v)) {
         user_open(snac, v);
@@ -303,6 +303,36 @@ int user_open_by_md5(snac *snac, const char *md5)
     return 0;
 }
 
+int user_persist(snac *snac, int publish)
+/* store user */
+{
+    xs *fn  = xs_fmt("%s/user.json", snac->basedir);
+    xs *bfn = xs_fmt("%s.bak", fn);
+    FILE *f;
+
+    rename(fn, bfn);
+
+    if ((f = fopen(fn, "w")) != NULL) {
+        xs_json_dump(snac->config, 4, f);
+        fclose(f);
+    }
+    else
+        rename(bfn, fn);
+
+    history_del(snac, "timeline.html_");
+
+    if (publish) {
+        xs *a_msg = msg_actor(snac);
+        xs *u_msg = msg_update(snac, a_msg);
+
+        enqueue_message(snac, u_msg);
+
+        enqueue_verify_links(snac);
+    }
+
+    return 0;
+}
+
 
 double mtime_nl(const char *fn, int *n_link)
 /* returns the mtime and number of links of a file or directory, or 0.0 */
@@ -355,12 +385,12 @@ int is_md5_hex(const char *md5)
 int index_add_md5(const char *fn, const char *md5)
 /* adds an md5 to an index */
 {
-    int status = 201; /* Created */
+    int status = HTTP_STATUS_CREATED;
     FILE *f;
 
     if (!is_md5_hex(md5)) {
         srv_log(xs_fmt("index_add_md5: bad md5 %s %s", fn, md5));
-        return 400;
+        return HTTP_STATUS_BAD_REQUEST;
     }
 
     pthread_mutex_lock(&data_mutex);
@@ -375,7 +405,7 @@ int index_add_md5(const char *fn, const char *md5)
         fclose(f);
     }
     else
-        status = 500;
+        status = HTTP_STATUS_INTERNAL_SERVER_ERROR;
 
     pthread_mutex_unlock(&data_mutex);
 
@@ -394,7 +424,7 @@ int index_add(const char *fn, const char *id)
 int index_del_md5(const char *fn, const char *md5)
 /* deletes an md5 from an index */
 {
-    int status = 404;
+    int status = HTTP_STATUS_NOT_FOUND;
     FILE *f;
 
     pthread_mutex_lock(&data_mutex);
@@ -411,7 +441,7 @@ int index_del_md5(const char *fn, const char *md5)
                    [yes: this breaks index_len()] */
                 fseek(f, -33, SEEK_CUR);
                 fwrite("-", 1, 1, f);
-                status = 200;
+                status = HTTP_STATUS_OK;
 
                 break;
             }
@@ -420,7 +450,7 @@ int index_del_md5(const char *fn, const char *md5)
         fclose(f);
     }
     else
-        status = 410;
+        status = HTTP_STATUS_GONE;
 
     pthread_mutex_unlock(&data_mutex);
 
@@ -660,7 +690,7 @@ int object_here(const char *id)
 int object_get_by_md5(const char *md5, xs_dict **obj)
 /* returns a stored object, optionally of the requested type */
 {
-    int status = 404;
+    int status = HTTP_STATUS_NOT_FOUND;
     xs *fn     = _object_fn_by_md5(md5, "object_get_by_md5");
     FILE *f;
 
@@ -669,7 +699,7 @@ int object_get_by_md5(const char *md5, xs_dict **obj)
         fclose(f);
 
         if (*obj)
-            status = 200;
+            status = HTTP_STATUS_OK;
     }
     else
         *obj = NULL;
@@ -689,7 +719,7 @@ int object_get(const char *id, xs_dict **obj)
 int _object_add(const char *id, const xs_dict *obj, int ow)
 /* stores an object */
 {
-    int status = 201; /* Created */
+    int status = HTTP_STATUS_CREATED; /* Created */
     xs *fn     = _object_fn(id);
     FILE *f;
 
@@ -697,10 +727,10 @@ int _object_add(const char *id, const xs_dict *obj, int ow)
         if (!ow) {
             /* object already here */
             srv_debug(1, xs_fmt("object_add object already here %s", id));
-            return 204; /* No content */
+            return HTTP_STATUS_NO_CONTENT;
         }
         else
-            status = 200;
+            status = HTTP_STATUS_OK;
     }
 
     if ((f = fopen(fn, "w")) != NULL) {
@@ -710,7 +740,7 @@ int _object_add(const char *id, const xs_dict *obj, int ow)
         fclose(f);
 
         /* does this object has a parent? */
-        char *in_reply_to = xs_dict_get(obj, "inReplyTo");
+        const char *in_reply_to = xs_dict_get(obj, "inReplyTo");
 
         if (!xs_is_null(in_reply_to) && *in_reply_to) {
             /* update the children index of the parent */
@@ -736,7 +766,7 @@ int _object_add(const char *id, const xs_dict *obj, int ow)
     }
     else {
         srv_log(xs_fmt("object_add error writing %s (errno: %d)", fn, errno));
-        status = 500;
+        status = HTTP_STATUS_INTERNAL_SERVER_ERROR;
     }
 
     srv_debug(1, xs_fmt("object_add %s %s %d", id, fn, status));
@@ -762,17 +792,18 @@ int object_add_ow(const char *id, const xs_dict *obj)
 int object_del_by_md5(const char *md5)
 /* deletes an object by its md5 */
 {
-    int status = 404;
+    int status = HTTP_STATUS_NOT_FOUND;
     xs *fn     = _object_fn_by_md5(md5, "object_del_by_md5");
 
     if (unlink(fn) != -1) {
-        status = 200;
+        status = HTTP_STATUS_OK;
 
         /* also delete associated indexes */
         xs *spec  = xs_dup(fn);
         spec      = xs_replace_i(spec, ".json", "*.idx");
         xs *files = xs_glob(spec, 0, 0);
-        char *p, *v;
+        char *p;
+        const char *v;
 
         p = files;
         while (xs_list_iter(&p, &v)) {
@@ -906,7 +937,7 @@ int object_parent(const char *md5, char *buf, int size)
 int object_admire(const char *id, const char *actor, int like)
 /* actor likes or announces this object */
 {
-    int status = 200;
+    int status = HTTP_STATUS_OK;
     xs *fn     = _object_fn(id);
 
     fn = xs_replace_i(fn, ".json", like ? "_l.idx" : "_a.idx");
@@ -1006,7 +1037,7 @@ int follower_add(snac *snac, const char *actor)
 
     snac_debug(snac, 2, xs_fmt("follower_add %s", actor));
 
-    return ret == -1 ? 500 : 200;
+    return ret == -1 ? HTTP_STATUS_INTERNAL_SERVER_ERROR : HTTP_STATUS_OK;
 }
 
 
@@ -1017,7 +1048,7 @@ int follower_del(snac *snac, const char *actor)
 
     snac_debug(snac, 2, xs_fmt("follower_del %s", actor));
 
-    return ret == -1 ? 404 : 200;
+    return ret == -1 ? HTTP_STATUS_NOT_FOUND : HTTP_STATUS_OK;
 }
 
 
@@ -1033,7 +1064,8 @@ xs_list *follower_list(snac *snac)
 {
     xs *list       = object_user_cache_list(snac, "followers", XS_ALL, 0);
     xs_list *fwers = xs_list_new();
-    char *p, *v;
+    char *p;
+    const char *v;
 
     /* resolve the list of md5 to be a list of actors */
     p = list;
@@ -1107,7 +1139,7 @@ int timeline_here(snac *snac, const char *md5)
 int timeline_get_by_md5(snac *snac, const char *md5, xs_dict **msg)
 /* gets a message from the timeline */
 {
-    int status = 404;
+    int status = HTTP_STATUS_NOT_FOUND;
     FILE *f    = NULL;
 
     xs *fn = timeline_fn_by_md5(snac, md5);
@@ -1117,14 +1149,14 @@ int timeline_get_by_md5(snac *snac, const char *md5, xs_dict **msg)
         fclose(f);
 
         if (*msg != NULL)
-            status = 200;
+            status = HTTP_STATUS_OK;
     }
 
     return status;
 }
 
 
-int timeline_del(snac *snac, char *id)
+int timeline_del(snac *snac, const char *id)
 /* deletes a message from the timeline */
 {
     /* delete from the user's caches */
@@ -1192,17 +1224,16 @@ int timeline_admire(snac *snac, const char *id, const char *admirer, int like)
 }
 
 
-xs_list *timeline_top_level(snac *snac, xs_list *list)
+xs_list *timeline_top_level(snac *snac, const xs_list *list)
 /* returns the top level md5 entries from this index */
 {
     xs_set seen;
-    xs_list *p;
-    xs_str *v;
+    const xs_str *v;
 
     xs_set_init(&seen);
 
-    p = list;
-    while (xs_list_iter(&p, &v)) {
+    int c = 0;
+    while (xs_list_next(list, &v, &c)) {
         char line[256] = "";
 
         strncpy(line, v, sizeof(line));
@@ -1281,7 +1312,7 @@ xs_str *_following_fn(snac *snac, const char *actor)
 int following_add(snac *snac, const char *actor, const xs_dict *msg)
 /* adds to the following list */
 {
-    int ret = 201; /* created */
+    int ret = HTTP_STATUS_CREATED;
     xs *fn = _following_fn(snac, actor);
     FILE *f;
     xs *p_object = NULL;
@@ -1290,11 +1321,11 @@ int following_add(snac *snac, const char *actor, const xs_dict *msg)
         /* object already exists; if it's of type Accept,
            the actor is already being followed and confirmed,
            so do nothing */
-        char *type = xs_dict_get(p_object, "type");
+        const char *type = xs_dict_get(p_object, "type");
 
         if (!xs_is_null(type) && strcmp(type, "Accept") == 0) {
             snac_debug(snac, 1, xs_fmt("following_add actor already confirmed %s", actor));
-            return 200;
+            return HTTP_STATUS_OK;
         }
     }
 
@@ -1310,7 +1341,7 @@ int following_add(snac *snac, const char *actor, const xs_dict *msg)
         link(actor_fn, fn);
     }
     else
-        ret = 500;
+        ret = HTTP_STATUS_INTERNAL_SERVER_ERROR;
 
     snac_debug(snac, 2, xs_fmt("following_add %s %s", actor, fn));
 
@@ -1331,7 +1362,7 @@ int following_del(snac *snac, const char *actor)
     fn = xs_replace_i(fn, ".json", "_a.json");
     unlink(fn);
 
-    return 200;
+    return HTTP_STATUS_OK;
 }
 
 
@@ -1349,14 +1380,14 @@ int following_get(snac *snac, const char *actor, xs_dict **data)
 {
     xs *fn = _following_fn(snac, actor);
     FILE *f;
-    int status = 200;
+    int status = HTTP_STATUS_OK;
 
     if ((f = fopen(fn, "r")) != NULL) {
         *data = xs_json_load(f);
         fclose(f);
     }
     else
-        status = 404;
+        status = HTTP_STATUS_NOT_FOUND;
 
     return status;
 }
@@ -1368,7 +1399,7 @@ xs_list *following_list(snac *snac)
     xs *spec = xs_fmt("%s/following/" "*.json", snac->basedir);
     xs *glist = xs_glob(spec, 0, 0);
     xs_list *p;
-    xs_str *v;
+    const xs_str *v;
     xs_list *list = xs_list_new();
 
     /* iterate the list of files */
@@ -1538,7 +1569,8 @@ void hide(snac *snac, const char *id)
 
         /* hide all the children */
         xs *chld = object_children(id);
-        char *p, *v;
+        char *p;
+        const char *v;
 
         p = chld;
         while (xs_list_iter(&p, &v)) {
@@ -1546,8 +1578,9 @@ void hide(snac *snac, const char *id)
 
             /* resolve to get the id */
             if (valid_status(object_get_by_md5(v, &co))) {
-                if ((v = xs_dict_get(co, "id")) != NULL)
-                    hide(snac, v);
+                const char *id = xs_dict_get(co, "id");
+                if (id != NULL)
+                    hide(snac, id);
             }
         }
     }
@@ -1563,7 +1596,7 @@ int is_hidden(snac *snac, const char *id)
 }
 
 
-int actor_add(const char *actor, xs_dict *msg)
+int actor_add(const char *actor, const xs_dict *msg)
 /* adds an actor */
 {
     return object_add_ow(actor, msg);
@@ -1573,7 +1606,7 @@ int actor_add(const char *actor, xs_dict *msg)
 int actor_get(const char *actor, xs_dict **data)
 /* returns an already downloaded actor */
 {
-    int status = 200;
+    int status = HTTP_STATUS_OK;
     xs_dict *d = NULL;
 
     if (xs_startswith(actor, srv_baseurl)) {
@@ -1587,10 +1620,10 @@ int actor_get(const char *actor, xs_dict **data)
                 *data = msg_actor(&user);
 
             user_free(&user);
-            return 200;
+            return HTTP_STATUS_OK;
         }
         else
-            return 404;
+            return HTTP_STATUS_NOT_FOUND;
     }
 
     /* read the object */
@@ -1603,7 +1636,7 @@ int actor_get(const char *actor, xs_dict **data)
     if (xs_is_null(xs_dict_get(d, "id")) || xs_is_null(xs_dict_get(d, "type"))) {
         srv_debug(1, xs_fmt("corrupted actor object %s", actor));
         d = xs_free(d);
-        return 404;
+        return HTTP_STATUS_NOT_FOUND;
     }
 
     if (data)
@@ -1619,7 +1652,7 @@ int actor_get(const char *actor, xs_dict **data)
 
     if (mtime(fn) + max_time < (double) time(NULL)) {
         /* actor data exists but also stinks */
-        status = 205; /* "205: Reset Content" "110: Response Is Stale" */
+        status = HTTP_STATUS_RESET_CONTENT; /* "110: Response Is Stale" */
     }
 
     return status;
@@ -1631,7 +1664,7 @@ int actor_get_refresh(snac *user, const char *actor, xs_dict **data)
 {
     int status = actor_get(actor, data);
 
-    if (status == 205 && user && !xs_startswith(actor, srv_baseurl))
+    if (status == HTTP_STATUS_RESET_CONTENT && user && !xs_startswith(actor, srv_baseurl))
         enqueue_actor_refresh(user, actor, 0);
 
     return status;
@@ -1687,17 +1720,18 @@ int limited(snac *user, const char *id, int cmd)
 void tag_index(const char *id, const xs_dict *obj)
 /* update the tag indexes for this object */
 {
-    xs_list *tags = xs_dict_get(obj, "tag");
+    const xs_list *tags = xs_dict_get(obj, "tag");
 
     if (is_msg_public(obj) && xs_type(tags) == XSTYPE_LIST && xs_list_len(tags) > 0) {
         xs *g_tag_dir = xs_fmt("%s/tag", srv_basedir);
 
         mkdirx(g_tag_dir);
 
-        xs_dict *v;
-        while (xs_list_iter(&tags, &v)) {
-            char *type = xs_dict_get(v, "type");
-            char *name = xs_dict_get(v, "name");
+        const xs_dict *v;
+        int ct = 0;
+        while (xs_list_next(tags, &v, &ct)) {
+            const char *type = xs_dict_get(v, "type");
+            const char *name = xs_dict_get(v, "name");
 
             if (!xs_is_null(type) && !xs_is_null(name) && strcmp(type, "Hashtag") == 0) {
                 while (*name == '#' || *name == '@')
@@ -1706,7 +1740,7 @@ void tag_index(const char *id, const xs_dict *obj)
                 if (*name == '\0')
                     continue;
 
-                name = xs_tolower_i(name);
+                name = xs_tolower_i((xs_str *)name);
 
                 xs *md5_tag   = xs_md5_hex(name, strlen(name));
                 xs *tag_dir   = xs_fmt("%s/%c%c", g_tag_dir, md5_tag[0], md5_tag[1]);
@@ -1729,7 +1763,7 @@ void tag_index(const char *id, const xs_dict *obj)
 }
 
 
-xs_list *tag_search(char *tag, int skip, int show)
+xs_list *tag_search(const char *tag, int skip, int show)
 /* returns the list of posts tagged with tag */
 {
     if (*tag == '#')
@@ -1757,7 +1791,7 @@ xs_val *list_maint(snac *user, const char *list, int op)
             xs *spec = xs_fmt("%s/list/" "*.id", user->basedir);
             xs *ls   = xs_glob(spec, 0, 0);
             int c = 0;
-            char *v;
+            const char *v;
 
             l = xs_list_new();
 
@@ -1783,7 +1817,7 @@ xs_val *list_maint(snac *user, const char *list, int op)
         {
             xs *lol = list_maint(user, NULL, 0);
             int c = 0;
-            xs_list *v;
+            const xs_list *v;
             int add = 1;
 
             /* check if this list name already exists */
@@ -1827,6 +1861,9 @@ xs_val *list_maint(snac *user, const char *list, int op)
 
                 fn = xs_replace_i(fn, ".lst", ".idx");
                 unlink(fn);
+
+                fn = xs_str_cat(fn, ".bak");
+                unlink(fn);
             }
         }
 
@@ -1888,7 +1925,7 @@ xs_val *list_content(snac *user, const char *list, const char *actor_md5, int op
 
     case 1: /** append actor to list **/
         if (actor_md5 != NULL) {
-            if (!index_in(fn, actor_md5))
+            if (!index_in_md5(fn, actor_md5))
                 index_add_md5(fn, actor_md5);
         }
 
@@ -1912,7 +1949,7 @@ xs_val *list_content(snac *user, const char *list, const char *actor_md5, int op
 void list_distribute(snac *user, const char *who, const xs_dict *post)
 /* distributes the post to all appropriate lists */
 {
-    char *id = xs_dict_get(post, "id");
+    const char *id = xs_dict_get(post, "id");
 
     /* if who is not set, use the attributedTo in the message */
     if (xs_is_null(who))
@@ -1924,7 +1961,7 @@ void list_distribute(snac *user, const char *who, const xs_dict *post)
         xs *spec  = xs_fmt("%s/list/" "*.lst", user->basedir);
         xs *ls    = xs_glob(spec, 0, 0);
         int c = 0;
-        char *v;
+        const char *v;
 
         while (xs_list_next(ls, &v, &c)) {
             /* is the actor in this list? */
@@ -1946,7 +1983,7 @@ static int _load_raw_file(const char *fn, xs_val **data, int *size,
                         const char *inm, xs_str **etag)
 /* loads a cached file */
 {
-    int status = 404;
+    int status = HTTP_STATUS_NOT_FOUND;
 
     if (fn) {
         double tm = mtime(fn);
@@ -1958,7 +1995,7 @@ static int _load_raw_file(const char *fn, xs_val **data, int *size,
             /* if if-none-match is set, check if it's the same */
             if (!xs_is_null(inm) && strcmp(e, inm) == 0) {
                 /* client has the newest version */
-                status = 304;
+                status = HTTP_STATUS_NOT_MODIFIED;
             }
             else {
                 /* newer or never downloaded; read the full file */
@@ -1969,7 +2006,7 @@ static int _load_raw_file(const char *fn, xs_val **data, int *size,
                     *data = xs_read(f, size);
                     fclose(f);
 
-                    status = 200;
+                    status = HTTP_STATUS_OK;
                 }
             }
 
@@ -2164,7 +2201,7 @@ void inbox_add(const char *inbox)
 void inbox_add_by_actor(const xs_dict *actor)
 /* collects an actor's shared inbox, if it has one */
 {
-    char *v;
+    const char *v;
 
     if (!xs_is_null(v = xs_dict_get(actor, "endpoints")) &&
         !xs_is_null(v = xs_dict_get(v, "sharedInbox"))) {
@@ -2182,7 +2219,7 @@ xs_list *inbox_list(void)
     xs *spec     = xs_fmt("%s/inbox/" "*", srv_basedir);
     xs *files    = xs_glob(spec, 0, 0);
     xs_list *p   = files;
-    xs_val *v;
+    const xs_val *v;
 
     while (xs_list_iter(&p, &v)) {
         FILE *f;
@@ -2210,7 +2247,7 @@ xs_str *_instance_block_fn(const char *instance)
     xs *s   = xs_replace(instance, "http:/" "/", "");
     xs *s1  = xs_replace(s, "https:/" "/", "");
     xs *l   = xs_split(s1, "/");
-    char *p = xs_list_get(l, 0);
+    const char *p = xs_list_get(l, 0);
     xs *md5 = xs_md5_hex(p, strlen(p));
 
     return xs_fmt("%s/block/%s", srv_basedir, md5);
@@ -2279,7 +2316,7 @@ int content_match(const char *file, const xs_dict *msg)
     xs *fn = xs_fmt("%s/%s", srv_basedir, file);
     FILE *f;
     int r = 0;
-    char *v = xs_dict_get(msg, "content");
+    const char *v = xs_dict_get(msg, "content");
 
     if (xs_type(v) == XSTYPE_STRING && *v) {
         if ((f = fopen(fn, "r")) != NULL) {
@@ -2328,8 +2365,8 @@ xs_list *content_search(snac *user, const char *regex,
 
     /* iterate all timelines simultaneously */
     xs_list *tls[3] = {0};
-    char *md5s[3]   = {0};
-    int c[3]        = {0};
+    const char *md5s[3] = {0};
+    int c[3] = {0};
 
     tls[0] = timeline_simple_list(user, "public", 0, XS_ALL);   /* public */
     tls[1] = timeline_instance_list(0, XS_ALL); /* instance */
@@ -2367,7 +2404,7 @@ xs_list *content_search(snac *user, const char *regex,
         if (newest == -1)
             break;
 
-        char *md5 = md5s[newest];
+        const char *md5 = md5s[newest];
 
         /* advance the chosen timeline */
         if (!xs_list_next(tls[newest], &md5s[newest], &c[newest]))
@@ -2386,7 +2423,7 @@ xs_list *content_search(snac *user, const char *regex,
         if (id == NULL || is_hidden(user, id))
             continue;
 
-        char *content = xs_dict_get(post, "content");
+        const char *content = xs_dict_get(post, "content");
 
         if (xs_is_null(content))
             continue;
@@ -2533,7 +2570,7 @@ xs_list *notify_list(snac *snac, int skip, int show)
             xs *spec = xs_fmt("%s/notify/" "*.json", snac->basedir);
             xs *lst  = xs_glob(spec, 1, 0);
             xs_list *p = lst;
-            char *v;
+            const char *v;
 
             while (xs_list_iter(&p, &v)) {
                 char *p = strrchr(v, '.');
@@ -2561,7 +2598,7 @@ int notify_new_num(snac *snac)
     int cnt = 0;
 
     xs_list *p = lst;
-    xs_str *v;
+    const xs_str *v;
 
     while (xs_list_iter(&p, &v)) {
         xs *id = xs_strip_i(xs_dup(v));
@@ -2583,7 +2620,7 @@ void notify_clear(snac *snac)
     xs *spec   = xs_fmt("%s/notify/" "*", snac->basedir);
     xs *lst    = xs_glob(spec, 0, 0);
     xs_list *p = lst;
-    xs_str *v;
+    const xs_str *v;
 
     while (xs_list_iter(&p, &v))
         unlink(v);
@@ -2639,7 +2676,7 @@ void enqueue_input(snac *snac, const xs_dict *msg, const xs_dict *req, int retri
 /* enqueues an input message */
 {
     xs *qmsg   = _new_qmsg("input", msg, retries);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", snac->basedir, ntid);
 
     qmsg = xs_dict_append(qmsg, "req", req);
@@ -2654,7 +2691,7 @@ void enqueue_shared_input(const xs_dict *msg, const xs_dict *req, int retries)
 /* enqueues an input message from the shared input */
 {
     xs *qmsg   = _new_qmsg("input", msg, retries);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
     qmsg = xs_dict_append(qmsg, "req", req);
@@ -2666,11 +2703,12 @@ void enqueue_shared_input(const xs_dict *msg, const xs_dict *req, int retries)
 
 
 void enqueue_output_raw(const char *keyid, const char *seckey,
-                        xs_dict *msg, xs_str *inbox, int retries, int p_status)
+                        const xs_dict *msg, const xs_str *inbox,
+                        int retries, int p_status)
 /* enqueues an output message to an inbox */
 {
     xs *qmsg   = _new_qmsg("output", msg, retries);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
     xs *ns = xs_number_new(p_status);
@@ -2690,7 +2728,8 @@ void enqueue_output_raw(const char *keyid, const char *seckey,
 }
 
 
-void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries, int p_status)
+void enqueue_output(snac *snac, const xs_dict *msg,
+                    const xs_str *inbox, int retries, int p_status)
 /* enqueues an output message to an inbox */
 {
     if (xs_startswith(inbox, snac->actor)) {
@@ -2698,13 +2737,14 @@ void enqueue_output(snac *snac, xs_dict *msg, xs_str *inbox, int retries, int p_
         return;
     }
 
-    char *seckey = xs_dict_get(snac->key, "secret");
+    const char *seckey = xs_dict_get(snac->key, "secret");
 
     enqueue_output_raw(snac->actor, seckey, msg, inbox, retries, p_status);
 }
 
 
-void enqueue_output_by_actor(snac *snac, xs_dict *msg, const xs_str *actor, int retries)
+void enqueue_output_by_actor(snac *snac, const xs_dict *msg,
+                            const xs_str *actor, int retries)
 /* enqueues an output message for an actor */
 {
     xs *inbox = get_actor_inbox(actor);
@@ -2716,11 +2756,11 @@ void enqueue_output_by_actor(snac *snac, xs_dict *msg, const xs_str *actor, int
 }
 
 
-void enqueue_email(xs_str *msg, int retries)
+void enqueue_email(const xs_str *msg, int retries)
 /* enqueues an email message to be sent */
 {
     xs *qmsg   = _new_qmsg("email", msg, retries);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
     qmsg = _enqueue_put(fn, qmsg);
@@ -2733,7 +2773,7 @@ void enqueue_telegram(const xs_str *msg, const char *bot, const char *chat_id)
 /* enqueues a message to be sent via Telegram */
 {
     xs *qmsg   = _new_qmsg("telegram", msg, 0);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
     qmsg = xs_dict_append(qmsg, "bot",      bot);
@@ -2748,7 +2788,7 @@ void enqueue_ntfy(const xs_str *msg, const char *ntfy_server, const char *ntfy_t
 /* enqueues a message to be sent via ntfy */
 {
     xs *qmsg   = _new_qmsg("ntfy", msg, 0);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", srv_basedir, ntid);
 
     qmsg = xs_dict_append(qmsg, "ntfy_server", ntfy_server);
@@ -2764,7 +2804,7 @@ void enqueue_message(snac *snac, const xs_dict *msg)
 /* enqueues an output message */
 {
     xs *qmsg   = _new_qmsg("message", msg, 0);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", snac->basedir, ntid);
 
     qmsg = _enqueue_put(fn, qmsg);
@@ -2807,7 +2847,7 @@ void enqueue_verify_links(snac *user)
 /* enqueues a link verification */
 {
     xs *qmsg   = _new_qmsg("verify_links", "", 0);
-    char *ntid = xs_dict_get(qmsg, "ntid");
+    const char *ntid = xs_dict_get(qmsg, "ntid");
     xs *fn     = xs_fmt("%s/queue/%s.json", user->basedir, ntid);
 
     qmsg = _enqueue_put(fn, qmsg);
@@ -2832,56 +2872,20 @@ void enqueue_actor_refresh(snac *user, const char *actor, int forward_secs)
 }
 
 
-void enqueue_request_replies(snac *user, const char *id)
-/* enqueues a request for the replies of a message */
-{
-    /* test first if this precise request is already in the queue */
-    xs *queue = user_queue(user);
-    xs_list *p = queue;
-    xs_str *v;
-
-    while (xs_list_iter(&p, &v)) {
-        xs *q_item = queue_get(v);
-
-        if (q_item != NULL) {
-            const char *type = xs_dict_get(q_item, "type");
-            const char *msg  = xs_dict_get(q_item, "message");
-
-            if (type && msg && strcmp(type, "request_replies") == 0 && strcmp(msg, id) == 0) {
-                /* don't requeue */
-                snac_debug(user, 1, xs_fmt("enqueue_request_replies already here %s", id));
-                return;
-            }
-        }
-    }
-
-    /* not there; enqueue the request with a small delay */
-    xs *qmsg = _new_qmsg("request_replies", id, 0);
-    xs *ntid = tid(10);
-    xs *fn   = xs_fmt("%s/queue/%s.json", user->basedir, ntid);
-
-    qmsg = xs_dict_set(qmsg, "ntid", ntid);
-
-    qmsg = _enqueue_put(fn, qmsg);
-
-    snac_debug(user, 2, xs_fmt("enqueue_request_replies %s", id));
-}
-
-
 int was_question_voted(snac *user, const char *id)
 /* returns true if the user voted in this poll */
 {
     xs *children = object_children(id);
     int voted = 0;
     xs_list *p;
-    xs_str *md5;
+    const xs_str *md5;
 
     p = children;
     while (xs_list_iter(&p, &md5)) {
         xs *obj = NULL;
 
         if (valid_status(object_get_by_md5(md5, &obj))) {
-            char *atto = get_atto(obj);
+            const char *atto = get_atto(obj);
             if (atto && strcmp(atto, user->actor) == 0 &&
                 !xs_is_null(xs_dict_get(obj, "name"))) {
                 voted = 1;
@@ -2901,7 +2905,7 @@ xs_list *user_queue(snac *snac)
     xs_list *list = xs_list_new();
     time_t t      = time(NULL);
     xs_list *p;
-    xs_val *v;
+    const xs_val *v;
 
     xs *fns = xs_glob(spec, 0, 0);
 
@@ -2930,7 +2934,7 @@ xs_list *queue(void)
     xs_list *list = xs_list_new();
     time_t t      = time(NULL);
     xs_list *p;
-    xs_val *v;
+    const xs_val *v;
 
     xs *fns = xs_glob(spec, 0, 0);
 
@@ -3006,7 +3010,7 @@ static void _purge_dir(const char *dir, int days)
         xs *spec  = xs_fmt("%s/" "*", dir);
         xs *list  = xs_glob(spec, 0, 0);
         xs_list *p;
-        xs_str *v;
+        const xs_str *v;
 
         p = list;
         while (xs_list_iter(&p, &v))
@@ -3032,7 +3036,7 @@ void purge_server(void)
     xs *spec = xs_fmt("%s/object/??", srv_basedir);
     xs *dirs = xs_glob(spec, 0, 0);
     xs_list *p;
-    xs_str *v;
+    const xs_str *v;
     int cnt = 0;
     int icnt = 0;
 
@@ -3041,7 +3045,7 @@ void purge_server(void)
     p = dirs;
     while (xs_list_iter(&p, &v)) {
         xs_list *p2;
-        xs_str *v2;
+        const xs_str *v2;
 
         {
             xs *spec2 = xs_fmt("%s/" "*.json", v);
@@ -3055,7 +3059,7 @@ void purge_server(void)
                 if (mtime_nl(v2, &n_link) < mt && n_link < 2) {
                     xs *s1    = xs_replace(v2, ".json", "");
                     xs *l     = xs_split(s1, "/");
-                    char *md5 = xs_list_get(l, -1);
+                    const char *md5 = xs_list_get(l, -1);
 
                     object_del_by_md5(md5);
                     cnt++;
@@ -3120,7 +3124,7 @@ void purge_server(void)
         xs *spec2 = xs_fmt("%s/" "*.idx", v);
         xs *files = xs_glob(spec2, 0, 0);
         xs_list *p2;
-        xs_str *v2;
+        const xs_str *v2;
 
         p2 = files;
         while (xs_list_iter(&p2, &v2)) {
@@ -3147,7 +3151,7 @@ void purge_user(snac *snac)
 /* do the purge for this user */
 {
     int priv_days, pub_days, user_days = 0;
-    char *v;
+    const char *v;
     int n;
 
     priv_days = xs_number_get(xs_dict_get(srv_config, "timeline_purge_days"));
@@ -3184,7 +3188,7 @@ void purge_user(snac *snac)
         xs *spec = xs_fmt("%s/list/" "*.idx", snac->basedir);
         xs *lol  = xs_glob(spec, 0, 0);
         int c = 0;
-        char *v;
+        const char *v;
 
         while (xs_list_next(lol, &v, &c)) {
             int gc = index_gc(v);
@@ -3202,7 +3206,8 @@ void purge_all(void)
 {
     snac snac;
     xs *list = user_list();
-    char *p, *uid;
+    char *p;
+    const char *uid;
 
     p = list;
     while (xs_list_iter(&p, &uid)) {
@@ -3256,7 +3261,7 @@ void srv_archive(const char *direction, const char *url, xs_dict *req,
         if (p_size && payload) {
             xs *payload_fn = NULL;
             xs *payload_fn_raw = NULL;
-            char *v = xs_dict_get(req, "content-type");
+            const char *v = xs_dict_get(req, "content-type");
 
             if (v && xs_str_in(v, "json") != -1) {
                 payload_fn = xs_fmt("%s/payload.json", dir);
@@ -3287,7 +3292,7 @@ void srv_archive(const char *direction, const char *url, xs_dict *req,
 
         if (b_size && body) {
             xs *body_fn = NULL;
-            char *v = xs_dict_get(headers, "content-type");
+            const char *v = xs_dict_get(headers, "content-type");
 
             if (v && xs_str_in(v, "json") != -1) {
                 body_fn = xs_fmt("%s/body.json", dir);
@@ -3356,7 +3361,7 @@ void srv_archive_error(const char *prefix, const xs_str *err,
 }
 
 
-void srv_archive_qitem(char *prefix, xs_dict *q_item)
+void srv_archive_qitem(const char *prefix, xs_dict *q_item)
 /* archives a q_item in the error folder */
 {
     xs *ntid = tid(0);
@@ -3368,3 +3373,69 @@ void srv_archive_qitem(char *prefix, xs_dict *q_item)
         fclose(f);
     }
 }
+
+
+t_announcement *announcement(const double after)
+/* returns announcement text or NULL if none exists or it is olde than "after" */
+{
+    static const long int MAX_SIZE = 2048;
+    static t_announcement a = {
+        .text = NULL,
+        .timestamp = 0.0,
+    };
+    static xs_str *fn = NULL;
+    if (fn == NULL)
+        fn = xs_fmt("%s/announcement.txt", srv_basedir);
+
+    const double ts = mtime(fn);
+
+    /* file does not exist or other than what was requested */
+    if (ts == 0.0 || ts <= after)
+        return NULL;
+
+    /* nothing changed, just return the current announcement */
+    if (a.text != NULL && ts <= a.timestamp)
+        return &a;
+
+    /* read and store new announcement */
+    FILE *f;
+
+    if ((f = fopen(fn, "r")) != NULL) {
+        fseek (f, 0, SEEK_END);
+        const long int length = ftell(f);
+
+        if (length > MAX_SIZE) {
+            /* this is probably unintentional */
+            srv_log(xs_fmt("announcement.txt too big: %ld bytes, max is %ld, ignoring.", length, MAX_SIZE));
+        }
+        else
+        if (length > 0) {
+            fseek (f, 0, SEEK_SET);
+            char *buffer = malloc(length + 1);
+            if (buffer) {
+                fread(buffer, 1, length, f);
+                buffer[length] = '\0';
+
+                free(a.text);
+                a.text = buffer;
+                a.timestamp = ts;
+            }
+            else {
+                srv_log("Error allocating memory for announcement");
+            }
+        }
+        else {
+            /* an empty file means no announcement */
+            free(a.text);
+            a.text = NULL;
+            a.timestamp = 0.0;
+        }
+
+        fclose (f);
+    }
+
+    if (a.text != NULL)
+        return &a;
+
+    return NULL;
+}
