@@ -75,7 +75,7 @@ xs_str *nodeinfo_2_0(void)
     int n_posts  = 0;
     xs *users = user_list();
     xs_list *p = users;
-    char *v;
+    const char *v;
     double now = (double)time(NULL);
 
     while (xs_list_iter(&p, &v)) {
@@ -125,10 +125,10 @@ static xs_str *greeting_html(void)
 
         /* does it have a %userlist% mark? */
         if (xs_str_in(s, "%userlist%") != -1) {
-            char *host = xs_dict_get(srv_config, "host");
+            const char *host = xs_dict_get(srv_config, "host");
             xs *list = user_list();
             xs_list *p = list;
-            xs_str *uid;
+            const xs_str *uid;
 
             xs_html *ul = xs_html_tag("ul",
                 xs_html_attr("class", "snac-user-list"));
@@ -171,14 +171,14 @@ int server_get_handler(xs_dict *req, const char *q_path,
 
     /* is it the server root? */
     if (*q_path == '\0') {
-        xs_dict *q_vars = xs_dict_get(req, "q_vars");
-        char *t = NULL;
+        const xs_dict *q_vars = xs_dict_get(req, "q_vars");
+        const char *t = NULL;
 
         if (xs_type(q_vars) == XSTYPE_DICT && (t = xs_dict_get(q_vars, "t"))) {
             /** search by tag **/
             int skip = 0;
             int show = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
-            char *v;
+            const char *v;
 
             if ((v = xs_dict_get(q_vars, "skip")) != NULL)
                 skip = atoi(v);
@@ -193,37 +193,41 @@ int server_get_handler(xs_dict *req, const char *q_path,
                 more = 1;
             }
 
-            char *accept = xs_dict_get(req, "accept");
+            const char *accept = xs_dict_get(req, "accept");
             if (!xs_is_null(accept) && strcmp(accept, "application/rss+xml") == 0) {
                 xs *link = xs_fmt("%s/?t=%s", srv_baseurl, t);
 
                 *body = timeline_to_rss(NULL, tl, link, link, link);
                 *ctype = "application/rss+xml; charset=utf-8";
             }
-            else
-                *body = html_timeline(NULL, tl, 0, skip, show, more, t, NULL, 0);
+            else {
+                xs *page = xs_fmt("?t=%s", t);
+                xs *title = xs_fmt(L("Search results for tag #%s"), t);
+                *body = html_timeline(NULL, tl, 0, skip, show, more, title, page, 0);
+            }
         }
         else
         if (xs_type(xs_dict_get(srv_config, "show_instance_timeline")) == XSTYPE_TRUE) {
             /** instance timeline **/
             xs *tl = timeline_instance_list(0, 30);
-            *body = html_timeline(NULL, tl, 0, 0, 0, 0, NULL, NULL, 0);
+            *body = html_timeline(NULL, tl, 0, 0, 0, 0,
+                L("Recent posts by users in this instance"), NULL, 0);
         }
         else
             *body = greeting_html();
 
         if (*body)
-            status = 200;
+            status = HTTP_STATUS_OK;
     }
     else
     if (strcmp(q_path, "/susie.png") == 0 || strcmp(q_path, "/favicon.ico") == 0 ) {
-        status = 200;
+        status = HTTP_STATUS_OK;
         *body  = xs_base64_dec(default_avatar_base64(), b_size);
         *ctype = "image/png";
     }
     else
     if (strcmp(q_path, "/.well-known/nodeinfo") == 0) {
-        status = 200;
+        status = HTTP_STATUS_OK;
         *ctype = "application/json; charset=utf-8";
         *body  = xs_fmt("{\"links\":["
             "{\"rel\":\"http:/" "/nodeinfo.diaspora.software/ns/schema/2.0\","
@@ -232,7 +236,7 @@ int server_get_handler(xs_dict *req, const char *q_path,
     }
     else
     if (strcmp(q_path, "/.well-known/host-meta") == 0) {
-        status = 200;
+        status = HTTP_STATUS_OK;
         *ctype = "application/xrd+xml";
         *body  = xs_fmt("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                 "<XRD>"
@@ -241,13 +245,13 @@ int server_get_handler(xs_dict *req, const char *q_path,
     }
     else
     if (strcmp(q_path, "/nodeinfo_2_0") == 0) {
-        status = 200;
+        status = HTTP_STATUS_OK;
         *ctype = "application/json; charset=utf-8";
         *body  = nodeinfo_2_0();
     }
     else
     if (strcmp(q_path, "/robots.txt") == 0) {
-        status = 200;
+        status = HTTP_STATUS_OK;
         *ctype = "text/plain";
         *body  = xs_str_new("User-agent: *\n"
                             "Disallow: /\n");
@@ -264,7 +268,7 @@ void httpd_connection(FILE *f)
 /* the connection processor */
 {
     xs *req;
-    char *method;
+    const char *method;
     int status   = 0;
     xs_str *body = NULL;
     int b_size   = 0;
@@ -274,7 +278,7 @@ void httpd_connection(FILE *f)
     xs *payload  = NULL;
     xs *etag     = NULL;
     int p_size   = 0;
-    char *p;
+    const char *p;
     int fcgi_id;
 
     if (p_state->use_fcgi)
@@ -356,10 +360,23 @@ void httpd_connection(FILE *f)
                         payload, p_size, &body, &b_size, &ctype);
 #endif
 
+    }
+    else
+    if (strcmp(method, "PATCH") == 0) {
+
+#ifndef NO_MASTODON_API
+        if (status == 0)
+            status = mastoapi_patch_handler(req, q_path,
+                        payload, p_size, &body, &b_size, &ctype);
+#endif
+
     }
     else
     if (strcmp(method, "OPTIONS") == 0) {
-        status = 200;
+        const char *methods = "OPTIONS, GET, HEAD, POST, PUT, DELETE";
+        headers = xs_dict_append(headers, "allow", methods);
+        headers = xs_dict_append(headers, "access-control-allow-methods", methods);
+        status = HTTP_STATUS_OK;
     }
     else
     if (strcmp(method, "DELETE") == 0) {
@@ -374,22 +391,22 @@ void httpd_connection(FILE *f)
     if (status == 0) {
         srv_archive_error("unattended_method", "unattended method", req, payload);
         srv_debug(1, xs_fmt("httpd_connection unattended %s %s", method, q_path));
-        status = 404;
+        status = HTTP_STATUS_NOT_FOUND;
     }
 
-    if (status == 403)
+    if (status == HTTP_STATUS_FORBIDDEN)
         body = xs_str_new("<h1>403 Forbidden</h1>");
 
-    if (status == 404)
+    if (status == HTTP_STATUS_NOT_FOUND)
         body = xs_str_new("<h1>404 Not Found</h1>");
 
-    if (status == 400 && body != NULL)
+    if (status == HTTP_STATUS_BAD_REQUEST && body != NULL)
         body = xs_str_new("<h1>400 Bad Request</h1>");
 
-    if (status == 303)
+    if (status == HTTP_STATUS_SEE_OTHER)
         headers = xs_dict_append(headers, "location", body);
 
-    if (status == 401) {
+    if (status == HTTP_STATUS_UNAUTHORIZED) {
         xs *www_auth = xs_fmt("Basic realm=\"@%s@%s snac login\"",
                                 body, xs_dict_get(srv_config, "host"));
 
@@ -407,9 +424,9 @@ void httpd_connection(FILE *f)
         headers = xs_dict_append(headers, "etag", etag);
 
     /* if there are any additional headers, add them */
-    xs_dict *more_headers = xs_dict_get(srv_config, "http_headers");
+    const xs_dict *more_headers = xs_dict_get(srv_config, "http_headers");
     if (xs_type(more_headers) == XSTYPE_DICT) {
-        char *k, *v;
+        const char *k, *v;
         int c = 0;
         while (xs_dict_next(more_headers, &k, &v, &c))
             headers = xs_dict_set(headers, k, v);
@@ -428,7 +445,7 @@ void httpd_connection(FILE *f)
     if (p_state->use_fcgi)
         xs_fcgi_response(f, status, headers, body, b_size, fcgi_id);
     else
-        xs_httpd_response(f, status, headers, body, b_size);
+        xs_httpd_response(f, status, http_status_text(status), headers, body, b_size);
 
     fclose(f);
 
@@ -586,7 +603,8 @@ static void *background_thread(void *arg)
 
         {
             xs *list = user_list();
-            char *p, *uid;
+            char *p;
+            const char *uid;
 
             /* process queues for all users */
             p = list;
@@ -660,6 +678,13 @@ srv_state *srv_state_op(xs_str **fname, int op)
 
     switch (op) {
     case 0: /* open for writing */
+
+#ifdef WITHOUT_SHM
+
+        errno = ENOTSUP;
+
+#else
+
         if ((fd = shm_open(*fname, O_CREAT | O_RDWR, 0666)) != -1) {
             ftruncate(fd, sizeof(*ss));
 
@@ -670,6 +695,8 @@ srv_state *srv_state_op(xs_str **fname, int op)
             close(fd);
         }
 
+#endif
+
         if (ss == NULL) {
             /* shared memory error: just create a plain structure */
             srv_log(xs_fmt("warning: shm object error (%s)", strerror(errno)));
@@ -683,6 +710,13 @@ srv_state *srv_state_op(xs_str **fname, int op)
         break;
 
     case 1: /* open for reading */
+
+#ifdef WITHOUT_SHM
+
+        errno = ENOTSUP;
+
+#else
+
         if ((fd = shm_open(*fname, O_RDONLY, 0666)) != -1) {
             if ((ss = mmap(0, sizeof(*ss), PROT_READ, MAP_SHARED, fd, 0)) == MAP_FAILED)
                 ss = NULL;
@@ -690,6 +724,8 @@ srv_state *srv_state_op(xs_str **fname, int op)
             close(fd);
         }
 
+#endif
+
         if (ss == NULL) {
             /* shared memory error */
             srv_log(xs_fmt("error: shm object error (%s) server not running?", strerror(errno)));
@@ -707,9 +743,14 @@ srv_state *srv_state_op(xs_str **fname, int op)
         break;
 
     case 2: /* unlink */
+
+#ifndef WITHOUT_SHM
+
         if (*fname)
             shm_unlink(*fname);
 
+#endif
+
         break;
     }
 
